#include "StdAfx.h"
#include "Sample.h"
#include "SampleDlg.h"
#include "VideoWin.h"


CFilterGraph::CFilterGraph(int i, CWnd *pParent)
	: m_pGraph(NULL)
	, m_pGraphAdo(NULL)
	, m_pMediaControl(NULL)
	, m_pMediaControlAdo(NULL)
	, m_pVCapFilter(NULL)
	, m_pACapFilter(NULL)
	, m_iSourceDeviceIndex(i)
	, m_bNtscSystem(true)
	, m_bCifSize(false)
	, m_inputMode(MODE_REALTIME)
	, m_pParent(pParent)
{
	for (int i = 0; i < MAX_VIDEO_PER_DEVICE; ++i)
	{
		m_pVideoRender[i] = NULL;
		m_pVideoProp[i] = NULL;
		m_pSConfig[i] = NULL;
	}
}


CFilterGraph::~CFilterGraph(void)
{
	if (m_pVCapFilter)
	{
		TeardownVideoGraph();
	}

	if (m_pACapFilter)
	{
		TeardownAudioGraph();
	}
}

HRESULT CFilterGraph::PlayVideoGraph(CComPtr<IBaseFilter> pVCapFilter)
{
	HRESULT hr = E_FAIL;

	// Create the filter graph
	hr = CoCreateInstance (CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **) &m_pGraph);
	if (SUCCEEDED(hr))
	{
		m_pVCapFilter = pVCapFilter;
		if (m_pVCapFilter)
		{
			CString sSrcFilterName;
			sSrcFilterName.Format(_T("%s%d"), VideoName, m_iSourceDeviceIndex + 1);

			hr = m_pGraph->AddFilter(m_pVCapFilter, sSrcFilterName);
			if (SUCCEEDED(hr))
			{
				hr = m_pVCapFilter->QueryInterface(IID_IAMAnalogVideoDecoder, (void**)&m_pVdoDec);
				if (FAILED(hr))
					return hr;

				hr = m_pVdoDec->put_TVFormat(m_bNtscSystem ? AnalogVideo_NTSC_M : AnalogVideo_PAL_B);

				for (int i = 0; i < static_cast<CSampleDlg*>(m_pParent)->GetInputModeCount(); ++i)
				{
					hr = CoCreateInstance(CLSID_VideoRenderer, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (LPVOID *)&m_pVideoRender[i]);
					if (SUCCEEDED(hr))
					{
						CString sRenderFilterName;
						sRenderFilterName.Format(_T("%s%d"), _T("Video Renderer"), i + 1);

						hr = m_pGraph->AddFilter(m_pVideoRender[i], sRenderFilterName);
						if (SUCCEEDED(hr))
						{
							// find the two pins and connect them
							IPin *pOut = GetOutPin(m_pVCapFilter, i);
							IPin *pIn = GetInPin(m_pVideoRender[i], 0);
							if (pOut && pIn)
							{
								hr = pOut->QueryInterface(IID_IAMStreamConfig, (PVOID *)&m_pSConfig[i]);
								if (SUCCEEDED(hr))
								{
									int iCount = 0, iSize = 0;
									hr = m_pSConfig[i]->GetNumberOfCapabilities(&iCount, &iSize);
									bool bFindIt = false;

									if (iSize == sizeof(VIDEO_STREAM_CONFIG_CAPS))
									{

										for (int iFormat = 0; iFormat < iCount; iFormat++)
										{
											VIDEO_STREAM_CONFIG_CAPS scc;
											AM_MEDIA_TYPE *pmt;
											hr = m_pSConfig[i]->GetStreamCaps(iFormat, &pmt, (BYTE*)&scc);
											if (SUCCEEDED(hr))
											{
												if (pmt->formattype == FORMAT_VideoInfo && pmt->subtype == MEDIASUBTYPE_YUY2)
												{

													VIDEOINFOHEADER *pVih =  reinterpret_cast<VIDEOINFOHEADER*>(pmt->pbFormat);
													BITMAPINFOHEADER *bmiHeader = &pVih->bmiHeader;

													if (bmiHeader->biWidth == _VideoWidth[m_bCifSize][m_bNtscSystem] &&
														bmiHeader->biHeight == _VideoHeight[m_bCifSize][m_bNtscSystem])
													{
														hr = m_pSConfig[i]->SetFormat(pmt);
														if (SUCCEEDED(hr))
														{
															bFindIt = true;
															OutputDebugString(_T("SetFormat Success\n"));
														}
													}
												}

												DeleteMediaType(pmt);
											}
										}
									}

									if (bFindIt == false)
									{
										hr = E_FAIL;
										return hr;
									}
								}

								hr = m_pGraph->Connect(pOut, pIn);
								if (SUCCEEDED(hr))
								{
									hr = m_pVideoRender[i]->QueryInterface(IID_IVideoWindow, (void **)&m_pVideoProp[i]);
									if (SUCCEEDED(hr))
									{
										int iCount = static_cast<CSampleDlg*>(m_pParent)->GetInputModeCount();

										CVideoWin *pVideoWin = static_cast<CSampleDlg*>(m_pParent)->m_pVideoWin[m_iSourceDeviceIndex * iCount + i];
										hr = m_pVideoProp[i]->put_Owner((OAHWND)pVideoWin->m_hWnd);
										hr = m_pVideoProp[i]->put_MessageDrain((OAHWND)pVideoWin->m_hWnd);
										hr = m_pVideoProp[i]->put_WindowStyle(WS_CHILD | WS_CLIPCHILDREN);
										hr = m_pVideoProp[i]->put_Visible(OATRUE);

										CRect rc;
										pVideoWin->GetClientRect(&rc);
										hr = m_pVideoProp[i]->SetWindowPosition(0, 0, rc.Width(), rc.Height());
									}
								}
							}
						}
					}
				}
			}
		}
	}

	if (SUCCEEDED(hr))
	{
		hr = m_pGraph->QueryInterface(IID_IMediaControl, (LPVOID *) &m_pMediaControl);
		if (SUCCEEDED(hr))
		{
			hr = m_pMediaControl->Run();
		}
	}

	return hr;
}


HRESULT CFilterGraph::PlayAudioGraph(CComPtr<IBaseFilter> pACapFilter)
{
	HRESULT hr = E_FAIL;

	// Create the filter graph
	hr = CoCreateInstance (CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **) &m_pGraphAdo);
	if (SUCCEEDED(hr))
	{
		m_pACapFilter = pACapFilter;
		if (m_pACapFilter)
		{
			CString sSrcFilterName;
			sSrcFilterName.Format(_T("%s%d"), AudioName, m_iSourceDeviceIndex + 1);

			hr = m_pGraphAdo->AddFilter(m_pACapFilter, sSrcFilterName);
			if (SUCCEEDED(hr))
			{
				IPin *pOut = GetOutPin(m_pACapFilter, 0);
				if (pOut)
				{
					hr = m_pGraphAdo->Render(pOut);
					if (SUCCEEDED(hr))
					{
						hr = m_pGraphAdo->QueryInterface(IID_IMediaControl, (LPVOID *) &m_pMediaControlAdo);
						if (SUCCEEDED(hr))
						{
							hr = m_pMediaControlAdo->Run();
						}
					}
				}
			}
		}
	}

	return hr;
}

HRESULT CFilterGraph::TeardownVideoGraph()
{
	HRESULT hr = E_FAIL;

	if (m_pMediaControl)
	{
		hr = m_pMediaControl->Stop();
	}

	if (m_pVCapFilter)
	{
		hr = m_pGraph->RemoveFilter(m_pVCapFilter);
	}

	for (int i = 0; i < MAX_VIDEO_PER_DEVICE; ++i)
	{
		if (m_pVideoRender[i])
		{
			hr = m_pGraph->RemoveFilter(m_pVideoRender[i]);
		}

		m_pVideoRender[i].Release();
		m_pVideoProp[i].Release();
		m_pSConfig[i].Release();
	}

	m_pVCapFilter.Release();
	m_pVdoDec.Release();
	m_pMediaControl.Release();
	m_pGraph.Release();

	return hr;
}

HRESULT CFilterGraph::TeardownAudioGraph()
{
	HRESULT hr = E_FAIL;

	if (m_pGraphAdo && m_pMediaControlAdo)
	{
		hr = m_pMediaControlAdo->Stop();
		if (SUCCEEDED(hr))
		{
			hr = m_pGraphAdo->RemoveFilter(m_pACapFilter);
		}

		m_pMediaControlAdo.Release();
		m_pGraphAdo.Release();
	}

	return hr;
}

void CFilterGraph::SetVideoFormat(bool bNtscSystem)
{
	m_bNtscSystem = bNtscSystem;
}

void CFilterGraph::SetVideoSize(bool bCifSize)
{
	m_bCifSize = bCifSize;
}

void CFilterGraph::SetInputMode(INPUT_MODE inputMode)
{
	m_inputMode = inputMode;
}

CComPtr<IVideoWindow> CFilterGraph::CheckVideoPropPointer(int i)
{
	if (i < MAX_VIDEO_PER_DEVICE)
	{
		return m_pVideoProp[i];
	}

	return NULL;
}