// PropSetDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Sample.h"
#include "SampleDlg.h"
#include "PropSetDlg.h"
#include "afxdialogex.h"

#define ID_CLEAR_WDT 1000
// CPropSetDlg dialog

IMPLEMENT_DYNAMIC(CPropSetDlg, CDialogEx)

CPropSetDlg::CPropSetDlg(CComPtr<IKsPropertySet> pPropSet /*=NULL*/)
	: CDialogEx(CPropSetDlg::IDD)
	, m_pPropSet(pPropSet)
	, m_GpioInStatus(0)
	, m_GpioOutStatus(0)
	, m_iWdtStatus(STOP)
{
	for (int i = 0; i < TOTAL_GPIO_PIN; ++i)
	{
		m_bCkGpioPin[i] = FALSE;
	}
}

CPropSetDlg::~CPropSetDlg()
{
	SetGPIOData(0x0000); //Reset GPIO output pin status

	SetWDT(WDT_STOP);
	KillTimer(ID_CLEAR_WDT);
}

void CPropSetDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);

	for (int i = 0; i < TOTAL_GPIO_PIN; ++i)
	{
		DDX_Check(pDX, IDC_PIN_CHECK0 + i, m_bCkGpioPin[i]);
	}
	DDX_Control(pDX, IDC_STATIC_HEX, m_Status);
	DDX_Control(pDX, IDC_STATIC_PIC_ID, m_PicId);
	DDX_Radio(pDX, IDC_RADIO_START, m_iWdtStatus);
}

BEGIN_MESSAGE_MAP(CPropSetDlg, CDialogEx)
	ON_BN_CLICKED(IDC_GET_ID_BTN, &CPropSetDlg::OnBnClickedGetIdBtn)
	ON_BN_CLICKED(IDC_GET_BTN, &CPropSetDlg::OnBnClickedGetBtn)
	
	ON_COMMAND_RANGE(IDC_RADIO_START, IDC_RADIO_TEST, OnWDT)
	ON_COMMAND_RANGE(IDC_PIN_CHECK8, IDC_PIN_CHECK15, OnSetOutPin)

	ON_WM_TIMER()
END_MESSAGE_MAP()


// CPropSetDlg message handlers


void CPropSetDlg::OnBnClickedGetIdBtn()
{
	// TODO: Add your control notification handler code here
	UpdateProtectedId();
}


void CPropSetDlg::OnBnClickedGetBtn()
{
	// TODO: Add your control notification handler code here
	UpdateGPIOStatus();
}

void CPropSetDlg::OnWDT(UINT uID)
{
	UpdateData(TRUE);

	KillTimer(ID_CLEAR_WDT);

	if (uID == IDC_RADIO_START)
	{
		SetWDT(WDT_START);

		SetTimer(ID_CLEAR_WDT, 1000, 0);
	}
	else if (uID == IDC_RADIO_STOP)
	{
		SetWDT(WDT_STOP);
	}
	else if (uID == IDC_RADIO_TEST)
	{
		SetWDT(WDT_START);
	}
}

void CPropSetDlg::OnSetOutPin(UINT uID)
{
	UINT uSetPin = uID - IDC_PIN_CHECK8;

	UpdateData(TRUE);

	m_GpioOutStatus[uSetPin] = m_bCkGpioPin[uSetPin + TOTAL_GPIO_PIN_IN];

	SetGPIOData(m_GpioOutStatus.to_ulong() << 8);
}

BOOL CPropSetDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// TODO:  Add extra initialization here
	
	InitializeGPIO();

	UpdateProtectedId();
	UpdateGPIOStatus();

	SetGPIOData(0x0000); //Reset GPIO output pin status

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}


bool CPropSetDlg::InitializeGPIO()
{
	if (m_pPropSet == NULL)
	{
		return false;
	}

	HRESULT hr;
	DWORD	dwTypeSupport, dwID;
	BYTE	byInstanceData[sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY)];
	TECHWELL_CUSTOM_CONTROL_S packet;
	packet.wGpioData = 0xFF00;

	WCHAR msg[30] = L"";
	wsprintf(msg, L"iiiii %x", packet.wGpioData);
	OutputDebugString(msg);

	dwID = KSPROPERTY_CUSTOM_INIT_GPIO;
	hr = m_pPropSet->Set( PROPERTYSETID_CUSTOM,
		dwID,
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S),
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S));
	wsprintf(msg, L"iiiii1 %x", packet.wGpioData);
	OutputDebugString(msg);
	return true;
}

bool CPropSetDlg::GetGPIOData(WORD* data)
{
	if (m_pPropSet == NULL)
	{
		return false;
	}

	HRESULT hr;
	DWORD	dwTypeSupport, dwID, ulLen = 0;
	BYTE	byInstanceData[sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY)];
	TECHWELL_CUSTOM_CONTROL_S packet;

	dwID = KSPROPERTY_CUSTOM_GPIO;
	hr = m_pPropSet->QuerySupported(PROPERTYSETID_CUSTOM
		, dwID, &dwTypeSupport);

	hr = m_pPropSet->Get(PROPERTYSETID_CUSTOM,
		dwID,
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S),
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S),
		&ulLen);
	if(FAILED(hr))
	{
		OutputDebugString(L"get failed");
	}
	Sleep(10);
	*data = packet.wGpioData;
	return true;
}

bool CPropSetDlg::GetPICData(BYTE* byData)
{
	if (m_pPropSet == NULL)
	{
		return false;
	}

	HRESULT hr;
	DWORD	dwTypeSupport, dwID, ulLen = 0;
	BYTE	byInstanceData[sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY)];
	TECHWELL_CUSTOM_CONTROL_S packet;

	dwID = KSPROPERTY_READ_PIC_ID;
	hr = m_pPropSet->QuerySupported(PROPERTYSETID_CUSTOM
		, dwID, &dwTypeSupport);

	hr = m_pPropSet->Get(PROPERTYSETID_CUSTOM,
		dwID,
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S),
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S),
		&ulLen);
	if(FAILED(hr))
	{
		OutputDebugString(L"get failed");
	}
	Sleep(10);
	*byData = packet.byPicId;
	return true;
}

bool CPropSetDlg::SetGPIOData(WORD data)
{
	if (m_pPropSet == NULL)
	{
		return false;
	}

	HRESULT hr;
	DWORD	dwTypeSupport, dwID;
	BYTE	byInstanceData[sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY)];
	TECHWELL_CUSTOM_CONTROL_S packet;
	packet.wGpioData = data;

	dwID = KSPROPERTY_CUSTOM_GPIO;
	hr = m_pPropSet->QuerySupported(PROPERTYSETID_CUSTOM
		, dwID, &dwTypeSupport);
	hr = m_pPropSet->Set( PROPERTYSETID_CUSTOM,
		dwID,
		&byInstanceData,
		sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY),
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S));
	return true;
}

bool CPropSetDlg::SetWDT(BYTE byData)
{
	if (m_pPropSet == NULL)
	{
		return false;
	}

	HRESULT hr;
	DWORD	dwTypeSupport, dwID;
	BYTE	byInstanceData[sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY)];
	TECHWELL_CUSTOM_CONTROL_S packet;
	packet.byWDTAction = byData;

	dwID = KSPROPERTY_SET_WDT;
	hr = m_pPropSet->QuerySupported(PROPERTYSETID_CUSTOM
		, dwID, &dwTypeSupport);

	hr = m_pPropSet->Set( PROPERTYSETID_CUSTOM,
		dwID,
		&byInstanceData,
		sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY),
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S));

	return true;
}

void CPropSetDlg::OnTimer(UINT_PTR nIDEvent)
{
	// TODO: Add your message handler code here and/or call default
	if (nIDEvent == ID_CLEAR_WDT)
	{
		KillTimer(ID_CLEAR_WDT);

		SetWDT(WDT_CLEAR);

		SetTimer(ID_CLEAR_WDT, 1000, 0);
	}

	CDialogEx::OnTimer(nIDEvent);
}

void CPropSetDlg::UpdateGPIOStatus()
{
	WORD wData = 0;
	if (GetGPIOData(&wData))
	{
		m_GpioInStatus = wData;

		for (int i = 0; i < TOTAL_GPIO_PIN / 2; ++i)
		{
			if(i < TOTAL_GPIO_PIN / 2)
				m_bCkGpioPin[i] = (wData >> i ) & 0x0001;
			else
				m_GpioOutStatus[i] = (wData >> i ) & 0x0001;

		}
	}

	TCHAR szHex[16];
	wsprintf(szHex, _T("0x%02X%02X"), m_GpioOutStatus.to_ulong(), m_GpioInStatus.to_ulong());
	m_Status.SetWindowText(szHex);

	UpdateData(FALSE);
}

void CPropSetDlg::UpdateProtectedId()
{
	BYTE byData = 0;
	if (GetPICData(&byData))
	{
		TCHAR szId[16];
		wsprintf(szId, _T("0x%02X"), byData);
		m_PicId.SetWindowText(szId);
	}

	UpdateData(FALSE);
}