﻿
// SampleDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Sample.h"
#include "SampleDlg.h"
#include "afxdialogex.h"
#include "PropSetDlg.h"
#include "InputModeDlg.h"
#include "SetValidChannelDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CAboutDlg dialog used for App About

class CAboutDlg : public CDialogEx
{
public:
	CAboutDlg();

// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialogEx(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialogEx)
END_MESSAGE_MAP()


// CSampleDlg dialog

CSampleDlg::CSampleDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CSampleDlg::IDD, pParent)
	, m_iACapFilterIndexNow(INT_MAX)
	, m_bFullScreen(false)
	, m_iIdxWinInFullScreen(INT_MAX)
	, m_bNtscSystem(true)
	, m_bCifSize(false)
	, m_inputMode(MODE_REALTIME)
	, m_nTotalChannel(8)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	CoInitialize(NULL);

	for (int i = 0; i < MAX_SOURCE_DEVICE; ++i)
	{
		m_pVCapFilter[i] = NULL;
		m_pACapFilter[i] = NULL;
		m_pPropSet[i] = NULL;
		m_pFilterGraph[i] = nullptr;
	}

	for (int i = 0; i < MAX_VIDEO_WIN; ++i)
	{
		m_pVideoWin[i] = nullptr;
	}

	m_mapInputMode[MODE_REALTIME] = 1;
	m_mapInputMode[MODE_SWITCH_2] = 2;
	m_mapInputMode[MODE_SWITCH_4] = 4;
}

void CSampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CSampleDlg, CDialogEx)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_COMMAND(ID_EXIT, &CSampleDlg::OnExit)
	ON_WM_DESTROY()
	ON_WM_SIZE()

	ON_COMMAND_RANGE(ID_LIVE_AUDIO1, ID_LIVE_AUDIO8, &CSampleDlg::OnLiveAudio)
	ON_COMMAND_RANGE(ID_FORMAT_NTSC, ID_FORMAT_PAL, &CSampleDlg::OnSwitchSystem)
	ON_COMMAND_RANGE(ID_SIZE_CIF, ID_SIZE_D1, &CSampleDlg::OnSwitchSize)
	ON_WM_LBUTTONDBLCLK()
	
	ON_MESSAGE(WM_FULLSCREEN, CSampleDlg::OnFullScreen)

	ON_COMMAND(ID_GPIO, &CSampleDlg::OnGpioDlg)
	ON_COMMAND(ID_INPUTMODE, &CSampleDlg::OnSetInputModeDlg)
	ON_COMMAND(ID_ENCHANNEL, &CSampleDlg::OnSetEnableChannelDlg)	
	ON_COMMAND(ID_DEINTERLACE_ENABLE, &CSampleDlg::OnDeinterlaceEnable)
END_MESSAGE_MAP()


// CSampleDlg message handlers

BOOL CSampleDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		BOOL bNameValid;
		CString strAboutMenu;
		bNameValid = strAboutMenu.LoadString(IDS_ABOUTBOX);
		ASSERT(bNameValid);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	//// TODO: Add extra initialization here
	HRESULT hr = EnumDevice();
	if (FAILED(hr))
	{
		MessageBox(_T("Don't have any card in your system!"), _T("Error"), MB_OK);

		OnOK();
		return FALSE;
	}

	// Initialize property set
	for (int i = 0 ; i < MAX_SOURCE_DEVICE; ++i)
	{
		if (m_pVCapFilter[i])
		{
			hr = m_pVCapFilter[i]->QueryInterface(IID_IKsPropertySet, (void**)&m_pPropSet[i]);
			if (FAILED(hr))
			{
				MessageBox(_T("Failed to get KsPropertySet"), _T("Error"), MB_OK);
				return FALSE;
			}
		}
	}

	InitVideoWin();

	TCHAR szPath[MAX_PATH] = {0};
	CString sFile;
	if (SUCCEEDED(SHGetFolderPath(NULL, CSIDL_LOCAL_APPDATA, NULL, 0, szPath)))
	{
		sFile = szPath;
		sFile += _T("\\Setting.dat");
	}
	else
	{
		sFile = _T("Setting.dat");
	}

	m_bNtscSystem = (GetPrivateProfileInt(_T("Demo"), _T("Format"), 0, sFile) != FALSE);
	m_bCifSize = (GetPrivateProfileInt(_T("Demo"), _T("Size"), 0, sFile) != FALSE);
	m_inputMode = static_cast<INPUT_MODE>(GetPrivateProfileInt(_T("Demo"), _T("Mode"), 0, sFile));

	if (m_inputMode == MODE_SWITCH_2)
		m_nTotalChannel = m_nTotalChannel * 2;
	else if (m_inputMode == MODE_SWITCH_4)
		m_nTotalChannel = m_nTotalChannel * 4;

	int iTotalWinIdx = 0;
	for (int i = 0; i < MAX_SOURCE_DEVICE; ++i)
	{
		if (m_pVCapFilter[i])
		{
			m_pFilterGraph[i] = new CFilterGraph(i, this);
			if (m_pFilterGraph[i])
			{
				m_pFilterGraph[i]->SetVideoFormat(m_bNtscSystem);
				m_pFilterGraph[i]->SetVideoSize(m_bCifSize);
				m_pFilterGraph[i]->SetInputMode(m_inputMode);

				hr = m_pFilterGraph[i]->PlayVideoGraph(m_pVCapFilter[i]);
				if (SUCCEEDED(hr))
				{
					for (int j = 0; j < MAX_VIDEO_PER_DEVICE; ++j)
					{
						CComPtr<IVideoWindow> pVideoProp = m_pFilterGraph[i]->CheckVideoPropPointer(j);
						if (pVideoProp)
						{
							m_pVideoWin[iTotalWinIdx]->SetVideoProp(pVideoProp);
							++iTotalWinIdx;
						}
					}
				}
			}
		}
	}

	MoveMainWindow();
			
	CMenu *pMenu = GetMenu();
	for (int i = 0; i < MAX_SOURCE_DEVICE; ++i)
	{
		pMenu->EnableMenuItem(ID_LIVE_AUDIO1 + i, m_pACapFilter[i] != NULL ? MF_ENABLED : MF_GRAYED);
	}

	pMenu->CheckMenuItem(m_bNtscSystem ? ID_FORMAT_NTSC : ID_FORMAT_PAL, MF_CHECKED | MF_BYCOMMAND);
	pMenu->CheckMenuItem(m_bCifSize ? ID_SIZE_CIF : ID_SIZE_D1, MF_CHECKED | MF_BYCOMMAND);

	pMenu->CheckMenuItem(ID_DEINTERLACE_ENABLE, (GetDeinterlace() ? MF_CHECKED : MF_UNCHECKED)| MF_BYCOMMAND);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CSampleDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialogEx::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSampleDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSampleDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}



void CSampleDlg::OnExit()
{
	// TODO: Add your command handler code here

	//Video Part
	for (int i = 0; i < MAX_VIDEO_WIN; ++i)
	{
		if (m_pVideoWin[i])
		{
			m_pVideoWin[i]->DestroyWindow();
		}

		SAFE_DELETE(m_pVideoWin[i]);
	}

	for (int i = 0; i < MAX_SOURCE_DEVICE; ++i)
	{
		SAFE_DELETE(m_pFilterGraph[i]);

		m_pVCapFilter[i].Release();
		m_pACapFilter[i].Release();
		m_pPropSet[i].Release();
	}

	CoUninitialize();

	TCHAR szPath[MAX_PATH] = {0};
	CString sFile;
	if (SUCCEEDED(SHGetFolderPath(NULL, CSIDL_LOCAL_APPDATA, NULL, 0, szPath)))
	{
		sFile = szPath;
		sFile += _T("\\Setting.dat");
	}
	else
	{
		sFile = _T("Setting.dat");
	}

	TCHAR keyVal[30] = {0};

	wsprintf(keyVal, _T("%d"), m_bNtscSystem != false);
	WritePrivateProfileString(_T("Demo"), _T("Format"), keyVal, sFile);

	wsprintf(keyVal, _T("%d"), m_bCifSize != false);
	WritePrivateProfileString(_T("Demo"), _T("Size"), keyVal, sFile);

	wsprintf(keyVal, _T("%d"), m_inputMode);
	WritePrivateProfileString(_T("Demo"), _T("Mode"), keyVal, sFile);

	m_mapInputMode.clear();

	OnOK();
}


HRESULT CSampleDlg::EnumDevice()
{
	HRESULT hr = E_FAIL;

	ICreateDevEnum *pCreateDevEnum = NULL;
	IEnumMoniker *pEnum = NULL;

	// Create the System Device Enumerator.
	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL,
		CLSCTX_INPROC_SERVER, IID_ICreateDevEnum, reinterpret_cast<void**>(&pCreateDevEnum));
	if (SUCCEEDED(hr))
	{
		// Create an enumerator for the video capture category.
		//hr = pCreateDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, &pEnum, 0);
		hr = pCreateDevEnum->CreateClassEnumerator(AM_KSCATEGORY_CAPTURE, &pEnum, 0);
	}
	pCreateDevEnum->Release();

	int iVdoIndex = 0, iAdoIndex = 0;
	IMoniker *pMoniker = NULL;
	while (hr = pEnum->Next(1, &pMoniker, NULL), hr == S_OK)
	{
		//Receive Friendly Name of Filter
		IPropertyBag *pPropBag;
		hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag,(void**)&pPropBag);

		VARIANT varName;
		VariantInit(&varName);

		hr = pPropBag->Read(L"FriendlyName", &varName, 0);
		if (SUCCEEDED(hr))
		{
			OutputDebugString(varName.bstrVal);
			OutputDebugString(_T("\n"));

			CString sDeviceName = varName.bstrVal;
			CString strVideoName;
			CString strAudioName;
			for(int i=0; i<MAX_SOURCE_DEVICE; i++)
			{
				strVideoName.Format(_T("%s%d"),VideoName,i+1);
				strAudioName.Format(_T("%s%d"),AudioName,i+1);

				if (sDeviceName.Find(strVideoName, 0) != -1)
				{
					if (iVdoIndex < MAX_SOURCE_DEVICE)
					{
						hr = pMoniker->BindToObject(0, 0, IID_IBaseFilter, (void**)&m_pVCapFilter[i]);
						if (SUCCEEDED(hr))
						{
							strVideoName.Format(_T("Create %s m_pVCapFilter[%d] iVdoIndex=%d OK!"), VideoName, i, iVdoIndex);
							OutputDebugString(strVideoName);
							iVdoIndex++;
							break;
						}
					}
				}
				else if (sDeviceName.Find(strAudioName, 0) != -1)
				{
					if (iAdoIndex < MAX_SOURCE_DEVICE)
					{
						hr = pMoniker->BindToObject(0, 0, IID_IBaseFilter, (void**)&m_pACapFilter[i]);
						if (SUCCEEDED(hr))
						{
							strAudioName.Format(_T("Create %s m_pACapFilter[%d] iAdoIndex =%d OK!"), AudioName, i, iAdoIndex);
							OutputDebugString(strAudioName);
							iAdoIndex++;
							break;
						}
					}
				}

			}


			VariantClear(&varName);
		}

		pPropBag->Release();
	}

	pMoniker->Release();

	if (iVdoIndex == 0)
	{
		hr = E_FAIL;
	}

	return hr;
}

BOOL CSampleDlg::GetDeinterlace()
{
	if (m_pPropSet[CTRL_DEVICE_ID] == NULL)
	{
		return false;
	}

	HRESULT hr;
	DWORD	dwTypeSupport, dwID, ulLen = 0;
	BYTE	byInstanceData[sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY)];
	TECHWELL_CUSTOM_CONTROL_S packet;

	dwID = KSPROPERTY_DELTERLACE;
	hr = m_pPropSet[CTRL_DEVICE_ID]->QuerySupported(PROPERTYSETID_CUSTOM
		, dwID, &dwTypeSupport);

	hr = m_pPropSet[CTRL_DEVICE_ID]->Get(PROPERTYSETID_CUSTOM,
		dwID,
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S),
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S),
		&ulLen);
	if(FAILED(hr))
	{
		OutputDebugString(L"get failed");
	}
	Sleep(10);
	return packet.uValue;
}

BOOL CSampleDlg::SetDeinterlace(BOOL bEnable)
{
	if (m_pPropSet[CTRL_DEVICE_ID] == NULL)
	{
		return false;
	}				   
	OutputDebugString(L"SetDeinterlace =============xxx");

	HRESULT hr;
	DWORD	dwTypeSupport, dwID;
	BYTE	byInstanceData[sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY)];
	TECHWELL_CUSTOM_CONTROL_S packet;
	packet.uValue = bEnable;

	dwID = KSPROPERTY_DELTERLACE;
	hr = m_pPropSet[CTRL_DEVICE_ID]->QuerySupported(PROPERTYSETID_CUSTOM
		, dwID, &dwTypeSupport);				    
	WCHAR msg[30] = L"";
	wsprintf(msg, L"SetDeinterlace 1hr%x",hr);
	OutputDebugString(msg);
	hr = m_pPropSet[CTRL_DEVICE_ID]->Set( PROPERTYSETID_CUSTOM,
		dwID,
		&byInstanceData,
		sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY),
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S));	 
	wsprintf(msg, L"SetDeinterlace 2hr%x",hr);
	OutputDebugString(msg);
	return true;
}

void CSampleDlg::OnDestroy()
{
	CDialogEx::OnDestroy();

	// TODO: Add your message handler code here
	OnExit();
}

void CSampleDlg::InitVideoWin()
{
	for (int i = 0; i < MAX_VIDEO_WIN; ++i)
	{
		m_pVideoWin[i] = new CVideoWin(i);
		m_pVideoWin[i]->Create(NULL, NULL, WS_VISIBLE | WS_CHILD | WS_BORDER, CRect(0, 0, 0, 0), this, 10000 + i);
	}
}

void CSampleDlg::OnSize(UINT nType, int cx, int cy)
{
	CDialogEx::OnSize(nType, cx, cy);

	// TODO: Add your message handler code here
	ResetVideoWinSize();
}

void CSampleDlg::ResetVideoWinSize()
{
	if (!m_pVideoWin[0])
		return;

	RECT rc;
	this->GetClientRect(&rc);

	int iWidth = rc.right - rc.left;
	int iHeight = rc.bottom - rc.top;

	if (m_bFullScreen)
	{
		for (int i = 0; i < MAX_VIDEO_WIN; ++i)
		{
			if (i == m_iIdxWinInFullScreen)
			{
				m_pVideoWin[i]->MoveWindow(-1, -1, iWidth, iHeight);
				m_pVideoWin[i]->MoveWindow(0, 0, iWidth, iHeight);
			}
			else
			{
				m_pVideoWin[i]->MoveWindow(0, 0, 0, 0);
			}
		}
	}
	else
	{
		int iScreenIndex = 3;
		if(m_nTotalChannel <= 9)
		{
			iScreenIndex = 3;	//9-Windows
		}
		else if(m_nTotalChannel <= 16)
		{
			iScreenIndex = 4;	//16-Windows
		}
		else if(m_nTotalChannel <= 36)
		{
			iScreenIndex = 6;	//36-Windows
		}

		int iMaxVideoWin = iScreenIndex * iScreenIndex;
		for (int i = 0; i < MAX_VIDEO_WIN; ++i)
		{
			if (m_pVideoWin[i] && (i < iMaxVideoWin))
			{
				m_pVideoWin[i]->ShowWindow(SW_SHOW);

				int iX = (iWidth / iScreenIndex) * (i % iScreenIndex);
				int iY = (iHeight / iScreenIndex) * (i / iScreenIndex);

				m_pVideoWin[i]->MoveWindow(iX + 1, iY + 1, iWidth / iScreenIndex, iHeight / iScreenIndex);
				m_pVideoWin[i]->MoveWindow(iX, iY, iWidth / iScreenIndex, iHeight / iScreenIndex);

				m_pVideoWin[i]->Invalidate();
			}
			else
			{
				m_pVideoWin[i]->ShowWindow(SW_HIDE);
			}
		}

		m_iIdxWinInFullScreen = INT_MAX;
	}
}

void CSampleDlg::OnLiveAudio(UINT nID)
{
	// TODO: Add your command handler code here
	
	UINT uIndex = nID - ID_LIVE_AUDIO1;
	
	CMenu* pMenu = GetMenu();

	if (m_iACapFilterIndexNow == uIndex) //stop original audio
	{
		pMenu->CheckMenuItem(nID, MF_UNCHECKED | MF_BYCOMMAND);

		m_pFilterGraph[m_iACapFilterIndexNow]->TeardownAudioGraph();

		m_iACapFilterIndexNow = INT_MAX;
	}
	else //change audio
	{
		if (m_iACapFilterIndexNow != INT_MAX) //live audio is playing
		{
			pMenu->CheckMenuItem(m_iACapFilterIndexNow + ID_LIVE_AUDIO1, MF_UNCHECKED | MF_BYCOMMAND);

			m_pFilterGraph[m_iACapFilterIndexNow]->TeardownAudioGraph();
		}

		m_iACapFilterIndexNow = uIndex;
		pMenu->CheckMenuItem(nID, MF_CHECKED | MF_BYCOMMAND);

		m_pFilterGraph[m_iACapFilterIndexNow]->PlayAudioGraph(m_pACapFilter[m_iACapFilterIndexNow]);
	}
}

void CSampleDlg::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default

	CDialogEx::OnLButtonDblClk(nFlags, point);
}

LRESULT CSampleDlg::OnFullScreen(WPARAM wParam, LPARAM lParam)
{
	m_bFullScreen = !m_bFullScreen;
	m_iIdxWinInFullScreen = static_cast<int>(wParam);

	ResetVideoWinSize();

	return 1;
}

void CSampleDlg::OnSwitchSize(UINT nID)
{
	HRESULT hr = E_FAIL;
	if (m_bCifSize != (nID == ID_SIZE_CIF))
	{
		CMenu* pMenu = GetMenu();
		pMenu->CheckMenuItem(m_bCifSize ? ID_SIZE_CIF : ID_SIZE_D1, MF_UNCHECKED | MF_BYCOMMAND);

		m_bCifSize = (nID == ID_SIZE_CIF);
		pMenu->CheckMenuItem(m_bCifSize ? ID_SIZE_CIF : ID_SIZE_D1, MF_CHECKED | MF_BYCOMMAND);

		RestartGraph();
	}
}
	
void CSampleDlg::OnSwitchSystem(UINT nID)
{
	HRESULT hr = E_FAIL;
	if (m_bNtscSystem != (nID == ID_FORMAT_NTSC))
	{
		CMenu* pMenu = GetMenu();
		pMenu->CheckMenuItem(m_bNtscSystem ? ID_FORMAT_NTSC : ID_FORMAT_PAL, MF_UNCHECKED | MF_BYCOMMAND);

		m_bNtscSystem = (nID == ID_FORMAT_NTSC);
		pMenu->CheckMenuItem(m_bNtscSystem ? ID_FORMAT_NTSC : ID_FORMAT_PAL, MF_CHECKED | MF_BYCOMMAND);

		RestartGraph();
	}
}

void CSampleDlg::MoveMainWindow()
{
	int X = GetSystemMetrics(SM_CXSCREEN);
	int Y = GetSystemMetrics(SM_CYSCREEN);

	MoveWindow(0, 0, X-1, Y-1);
	MoveWindow(0, 0, X, Y);
}

void CSampleDlg::OnGpioDlg()
{
	// TODO: Add your command handler code here
	CPropSetDlg dlg(m_pPropSet[CTRL_DEVICE_ID]);
	dlg.DoModal();
}

void CSampleDlg::RestartGraph()
{
	HRESULT hr = E_FAIL;

	for (int i = 0; i < MAX_VIDEO_WIN; ++i)
	{
		m_pVideoWin[i]->SetVideoProp(NULL);
	}

	for (int i = 0; i < MAX_SOURCE_DEVICE; ++i)
	{
		if (m_pVCapFilter[i])
		{
			hr = m_pFilterGraph[i]->TeardownVideoGraph();
		}
	}

	m_nTotalChannel = 8;
	if (m_inputMode == MODE_SWITCH_2)
		m_nTotalChannel = m_nTotalChannel * 2;
	else if (m_inputMode == MODE_SWITCH_4)
		m_nTotalChannel = m_nTotalChannel * 4;

	int iTotalWinIdx = 0;
	for (int i = 0; i < MAX_SOURCE_DEVICE; ++i)
	{
		if (m_pVCapFilter[i] && m_pFilterGraph[i])
		{
			m_pFilterGraph[i]->SetInputMode(m_inputMode);
			m_pFilterGraph[i]->SetVideoFormat(m_bNtscSystem);
			m_pFilterGraph[i]->SetVideoSize(m_bCifSize);

			hr = m_pFilterGraph[i]->PlayVideoGraph(m_pVCapFilter[i]);
			if (SUCCEEDED(hr))
			{
				for (int j = 0; j < MAX_VIDEO_PER_DEVICE; ++j)
				{
					CComPtr<IVideoWindow> pVideoProp = m_pFilterGraph[i]->CheckVideoPropPointer(j);
					if (pVideoProp)
					{
						m_pVideoWin[iTotalWinIdx]->SetVideoProp(pVideoProp);
						++iTotalWinIdx;
					}
				}
			}
		}
	}

	ResetVideoWinSize();
}

void CSampleDlg::OnSetInputModeDlg()
{
	// TODO: Add your command handler code here
	CInputModeDlg dlg(static_cast<int>(m_inputMode));
	
	if (dlg.DoModal() == IDOK)
	{
		HRESULT hr = E_FAIL;

		if (m_inputMode != static_cast<INPUT_MODE>(dlg.m_iInputMode))
		{
			m_inputMode = static_cast<INPUT_MODE>(dlg.m_iInputMode);

			RestartGraph();
		}
	}
}

void CSampleDlg::OnSetEnableChannelDlg()
{
	// TODO: Add your command handler code here
	CSetValidChannelDlg dlg(this);

	if (dlg.DoModal() == IDOK)
	{
	}
}

int CSampleDlg::GetInputModeCount()
{
	if (m_mapInputMode.count(m_inputMode) > 0)
	{
		return m_mapInputMode[m_inputMode];
	}

	return 0;
}

void CSampleDlg::OnDeinterlaceEnable()
{	  
	CMenu* pMenu = GetMenu();
	if(pMenu->GetMenuState(ID_DEINTERLACE_ENABLE,MF_BYCOMMAND)&MF_CHECKED)
	{
		SetDeinterlace(FALSE);
		pMenu->CheckMenuItem(ID_DEINTERLACE_ENABLE, MF_UNCHECKED|MF_BYCOMMAND);
	}
	else
	{
		SetDeinterlace(TRUE);
		pMenu->CheckMenuItem(ID_DEINTERLACE_ENABLE, MF_CHECKED|MF_BYCOMMAND);
	}
	
}
