
// SampleDlg.h : header file
//

#pragma once
#include "custom_property.h"
#include "FilterGraph.h"
#include "VideoWin.h"

class CFilterGraph;

#define MAX_SOURCE_DEVICE 8
#define MAX_VIDEO_WIN 36

static const TCHAR VideoName[] = _T("Video FC");
static const TCHAR AudioName[] = _T("Audio FC");

#define WIDTH_WIN 1024
#define HEIGHT_WIN 768

#define WM_FULLSCREEN WM_USER + 100

enum
{

	CAPTURE_WIDTH_D1_PAL = 704,
	CAPTURE_HEIGHT_D1_PAL = 576,
	CAPTURE_WIDTH_D1_NTSC = 704,
	CAPTURE_HEIGHT_D1_NTSC = 480,

	CAPTURE_WIDTH_CIF_PAL = 352,
	CAPTURE_HEIGHT_CIF_PAL = CAPTURE_HEIGHT_D1_PAL/2,
	CAPTURE_WIDTH_CIF_NTSC =  320,
	CAPTURE_HEIGHT_CIF_NTSC = CAPTURE_HEIGHT_D1_NTSC/2,

};
enum{
	F_PAL,
	F_NTSC,
	MAX_FORMAT
};

enum{
	SIZE_D1,
	SIZE_CIF,
	MAX_SIZE_TYPE
};
const int _VideoWidth[MAX_SIZE_TYPE][MAX_FORMAT] = {
	{CAPTURE_WIDTH_D1_PAL,CAPTURE_WIDTH_D1_NTSC},
	{CAPTURE_WIDTH_CIF_PAL,CAPTURE_WIDTH_CIF_NTSC},
};
const int _VideoHeight[MAX_SIZE_TYPE][MAX_FORMAT] = {
	{CAPTURE_HEIGHT_D1_PAL,CAPTURE_HEIGHT_D1_NTSC},
	{CAPTURE_HEIGHT_CIF_PAL,CAPTURE_HEIGHT_CIF_NTSC},
};

#define CTRL_DEVICE_ID	0

// CSampleDlg dialog
class CSampleDlg : public CDialogEx
{
// Construction
public:
	CSampleDlg(CWnd* pParent = NULL);	// standard constructor

	CVideoWin *m_pVideoWin[MAX_VIDEO_WIN];

	HRESULT EnumDevice();
	CComPtr<IBaseFilter> m_pVCapFilter[MAX_SOURCE_DEVICE];
	CComPtr<IBaseFilter> m_pACapFilter[MAX_SOURCE_DEVICE];
	CComPtr<IKsPropertySet> m_pPropSet[MAX_SOURCE_DEVICE];

// Dialog Data
	enum { IDD = IDD_SAMPLE_DIALOG };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

	CFilterGraph *m_pFilterGraph[MAX_SOURCE_DEVICE];

	void InitVideoWin();
	void ResetVideoWinSize();
	void MoveMainWindow();

	map<INPUT_MODE, int> m_mapInputMode;

	INPUT_MODE m_inputMode;
	int m_iACapFilterIndexNow;
	bool m_bFullScreen;
	int m_iIdxWinInFullScreen;

	bool m_bNtscSystem;
	bool m_bCifSize;
	BOOL m_bDeinterlace;

	int m_nTotalChannel;
	BOOL GetDeinterlace();
	BOOL SetDeinterlace(BOOL bEnable);

// Implementation
protected:
	HICON m_hIcon;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnExit();
	afx_msg void OnDestroy();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnLiveAudio(UINT nID);
	afx_msg void OnSwitchSystem(UINT nID);
	afx_msg void OnSwitchSize(UINT nID);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	LRESULT OnFullScreen(WPARAM wParam, LPARAM lParam);
	afx_msg void OnGpioDlg();
	afx_msg void OnSetInputModeDlg();
	afx_msg void OnSetEnableChannelDlg();

	int GetInputModeCount();
	void RestartGraph();
	afx_msg void OnDeinterlaceEnable();
};
