#pragma once

#include <map>
using namespace std;

#define MAX_VIDEO_PER_DEVICE 4
typedef enum { MODE_REALTIME, MODE_SWITCH_2, MODE_SWITCH_4} INPUT_MODE;

class CFilterGraph
{

	HRESULT GetPin( IBaseFilter * pFilter, PIN_DIRECTION dirrequired, int iNum, IPin **ppPin)
	{
		CComPtr< IEnumPins > pEnum;
		*ppPin = NULL;
		HRESULT hr = pFilter->EnumPins(&pEnum);
		if(FAILED(hr)) 
			return hr;

		ULONG ulFound;
		IPin *pPin;
		hr = E_FAIL;
		while(S_OK == pEnum->Next(1, &pPin, &ulFound))
		{
			PIN_DIRECTION pindir = (PIN_DIRECTION)3;
			pPin->QueryDirection(&pindir);
			if(pindir == dirrequired)
			{
				if(iNum == 0)
				{
					*ppPin = pPin;
					break;
				}
				iNum--;
			} // if
			pPin->Release();
		} // while

		return hr;
	}

	IPin * GetInPin( IBaseFilter * pFilter, int Num )
	{
		CComPtr< IPin > pComPin;
		GetPin(pFilter, PINDIR_INPUT, Num, &pComPin);
		return pComPin;
	}

	IPin * GetOutPin( IBaseFilter * pFilter, int Num )
	{
		CComPtr< IPin > pComPin;
		GetPin(pFilter, PINDIR_OUTPUT, Num, &pComPin);
		return pComPin;
	}

	CComPtr<IGraphBuilder> m_pGraph;
	CComPtr<IGraphBuilder> m_pGraphAdo;
	
	CComPtr<IBaseFilter> m_pVCapFilter;
	CComPtr<IBaseFilter> m_pACapFilter;

	CComPtr<IBaseFilter> m_pVideoRender[MAX_VIDEO_PER_DEVICE];

	CComPtr<IVideoWindow> m_pVideoProp[MAX_VIDEO_PER_DEVICE];
	CComPtr<IMediaControl> m_pMediaControl;
	CComPtr<IMediaControl> m_pMediaControlAdo;

	CComPtr<IAMAnalogVideoDecoder> m_pVdoDec;
	CComPtr<IAMStreamConfig> m_pSConfig[MAX_VIDEO_PER_DEVICE];

	CWnd *m_pParent;
	int m_iSourceDeviceIndex;
	bool m_bNtscSystem;
	bool m_bCifSize;
	INPUT_MODE m_inputMode;

public:
	CFilterGraph(int i, CWnd *pParent = NULL);
	~CFilterGraph(void);
	
	HRESULT PlayVideoGraph(CComPtr<IBaseFilter> pVCapFilter);
	HRESULT PlayAudioGraph(CComPtr<IBaseFilter> pACapFilter);

	HRESULT TeardownVideoGraph();
	HRESULT TeardownAudioGraph();

	void SetVideoFormat(bool bNtscSystem);
	void SetVideoSize(bool bCifSize);
	void SetInputMode(INPUT_MODE inputMode);
	CComPtr<IVideoWindow> CheckVideoPropPointer(int i);
};

