// SetValidChannelDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Sample.h"
#include "SampleDlg.h"
#include "SetValidChannelDlg.h"
#include "afxdialogex.h"


// CSetValidChannelDlg dialog

IMPLEMENT_DYNAMIC(CSetValidChannelDlg, CDialogEx)

CSetValidChannelDlg::CSetValidChannelDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CSetValidChannelDlg::IDD, pParent)
	, m_pParent(pParent)
{
	for (int i = 0; i < MAX_SOURCE_DEVICE; ++i)
	{
		m_pPropSet[i] = NULL;

		if (pParent)
		{
			m_pPropSet[i] = static_cast<CSampleDlg*>(pParent)->m_pPropSet[i];
		}
	}

	memset(m_bEnableCH, 0, MAX_VIDEO_PER_DEVICE);
}

CSetValidChannelDlg::~CSetValidChannelDlg()
{
}

void CSetValidChannelDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CBO_DEVICE_LIST, m_cboDeviceList);
	
	for (int i = 0; i < MAX_VIDEO_PER_DEVICE; ++i)
		DDX_Check(pDX, IDC_CK_VIDEO1 + i, m_bEnableCH[i]);
}


BEGIN_MESSAGE_MAP(CSetValidChannelDlg, CDialogEx)
	ON_BN_CLICKED(IDC_BTN_SET, &CSetValidChannelDlg::OnBnClickedBtnSet)
	ON_CBN_SELCHANGE(IDC_CBO_DEVICE_LIST, &CSetValidChannelDlg::OnCbnSelchangeCboDeviceList)

END_MESSAGE_MAP()


// CSetValidChannelDlg message handlers


void CSetValidChannelDlg::OnBnClickedBtnSet()
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);

	int iSel = m_cboDeviceList.GetCurSel();
	if (iSel != -1)
	{
		ULONG uValidChannel = GetChannelStatus(iSel);
		uValidChannel = uValidChannel & ~(0xF << (iSel * MAX_VIDEO_PER_DEVICE));

		for (int i = 0; i < MAX_VIDEO_PER_DEVICE; ++i)
		{
			 uValidChannel = uValidChannel | (m_bEnableCH[i] << (i + iSel * MAX_VIDEO_PER_DEVICE));
		}

		SetChannelStatus(iSel, uValidChannel);
	}
}


void CSetValidChannelDlg::OnCbnSelchangeCboDeviceList()
{
	// TODO: Add your control notification handler code here
	int iSel = m_cboDeviceList.GetCurSel();
	if (iSel != -1)
	{
		ULONG uValidChannel = GetChannelStatus(iSel);
		if (uValidChannel > 0)
		{
			for (int i = 0; i < MAX_VIDEO_PER_DEVICE; ++i)
			{
				m_bEnableCH[i] = (uValidChannel >> (i + iSel* MAX_VIDEO_PER_DEVICE)) & 0x01;
			}
		}

		UpdateData(FALSE);
	}
}


BOOL CSetValidChannelDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// TODO:  Add extra initialization here

	CString str;
	for (int i = 0; i < MAX_SOURCE_DEVICE; ++i)
	{
		str.Format(_T("Video Device Index #%02d"), i+1);
		m_cboDeviceList.AddString(str);
	}

	m_cboDeviceList.SetCurSel(0);
	OnCbnSelchangeCboDeviceList();

	int iCount = static_cast<CSampleDlg*>(m_pParent)->GetInputModeCount();
	for (int i = 0; i < MAX_VIDEO_PER_DEVICE; ++i)
	{
		GetDlgItem(IDC_CK_VIDEO1 + i)->EnableWindow(i < iCount);
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

ULONG CSetValidChannelDlg::GetChannelStatus(int iDevice)
{
	if (m_pPropSet[iDevice] == NULL)
	{
		OutputDebugString(_T("GetChannelStatus\n"));
		return 0;
	}

	HRESULT hr;
	DWORD	dwTypeSupport, dwID, ulLen = 0;
	BYTE	byInstanceData[sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY)];
	TECHWELL_CUSTOM_CONTROL_S packet;

	dwID = KSPROPERTY_VALID_VIDEO_CHANNEL;
	hr = m_pPropSet[iDevice]->QuerySupported(PROPERTYSETID_CUSTOM
		, dwID, &dwTypeSupport);
	if (SUCCEEDED(hr))
	{
		hr = m_pPropSet[iDevice]->Get(PROPERTYSETID_CUSTOM,
			dwID,
			&packet,
			sizeof(TECHWELL_CUSTOM_CONTROL_S),
			&packet,
			sizeof(TECHWELL_CUSTOM_CONTROL_S),
			&ulLen);

		CString str;
		str.Format(_T("GetChannelStatus(0x%08X)\n"), packet.ulValidVideoChannel);
		OutputDebugString(str);

	}

	return packet.ulValidVideoChannel;
}

bool CSetValidChannelDlg::SetChannelStatus(int iDevice, ULONG uValidChannel)
{
	if (m_pPropSet[iDevice] == NULL)
	{
		OutputDebugString(_T("SetChannelStatus\n"));
		return false;
	}

	HRESULT hr;
	DWORD	dwTypeSupport, dwID;
	BYTE	byInstanceData[sizeof(TECHWELL_CUSTOM_CONTROL_S) - sizeof(KSPROPERTY)];
	
	TECHWELL_CUSTOM_CONTROL_S packet;
	packet.ulValidVideoChannel = uValidChannel;

	dwID = KSPROPERTY_VALID_VIDEO_CHANNEL;
	hr = m_pPropSet[iDevice]->Set( PROPERTYSETID_CUSTOM,
		dwID,
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S),
		&packet,
		sizeof(TECHWELL_CUSTOM_CONTROL_S));

	CString str;
	str.Format(_T("SetChannelStatus(0x%08X)\n"), uValidChannel);
	OutputDebugString(str);

	return true;
}